/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Copyright 2005 Todd Kulesza
 *
 */

#include <config.h>

#include <string.h>
#include <curl/curl.h>

#include "plugin.h"

DrivelPlugin*
drivel_plugin_init (GModule *module)
{
	DrivelPlugin *plugin = drivel_plugin_new ();
	
	plugin->module = module;
	plugin->name = g_strdup ("LiveJournal");
	plugin->journals = g_array_new (TRUE, FALSE, sizeof (DrivelJournalPair *));
	plugin->journals = drivel_plugin_add_journal (plugin->journals,
						      "LiveJournal",
						      "http://www.livejournal.com");
	plugin->journals = drivel_plugin_add_journal (plugin->journals,
						      "DeadJournal",
						      "http://www.deadjournal.com");

	return plugin;
}

DrivelRequest*
drivel_plugin_build_login_request (const gchar *arg, ...)
{
	DrivelRequest *dr;
	va_list ap;
	const gchar *key, *value, *username, *server, *n_moods;
	gchar *version = g_strdup_printf ("GNOME-Drivel/%s", VERSION);

	username = server = n_moods = NULL;

	va_start (ap, arg);
	do
	{
		if ((key = va_arg (ap, gchar*)))
		{
			value = va_arg (ap, gchar*);
			if (!strcmp (key, "username"))
				username = value;
			else if (!strcmp (key, "server"))
				server = value;
			else if (!strcmp (key, "n_moods"))
				n_moods = value;
		}
	} while (key);
	
	dr = drivel_request_new_with_items (REQUEST_TYPE_LOGIN, 
					    REQUEST_PROTOCOL_POST, 
					    BLOG_API_LJ,
					    server,
					    g_strdup ("mode"), g_strdup ("login"),
					    g_strdup ("user"), curl_escape (username, 0),
					    g_strdup ("ver"), g_strdup ("1"),
					    g_strdup ("clientversion"), curl_escape (version, 0),
					    g_strdup ("getpickws"), g_strdup ("1"),
					    g_strdup ("getpickwurls"), g_strdup ("1"),
					    g_strdup ("getmenus"), g_strdup ("1"),
					    g_strdup ("getmoods"), g_strdup (n_moods),
					    NULL);

	return dr;
}
