/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Copyright 2002-2005 Todd Kulesza
 *
 * Authors:
 * 		Todd Kulesza <todd@dropline.net>
 */

#include <config.h>

#include <gnome.h>

#include "about.h"
#include "drivel.h"

static void
url_cb (GtkAboutDialog *about, const gchar *url, gpointer data)
{
  GError *error = NULL;
  DrivelClient *dc = (DrivelClient *)data;

  gnome_url_show (url, &error);
  if (error)
    {
      display_error_dialog (dc, 
			    _("There was a problem viewing the website"), 
			    error->message);
      g_error_free (error);
    }

  return;
}

void
about_show (DrivelClient *dc)
{
  static GdkPixbuf *logo = NULL;
  const gchar *authors[] = 	
    {
      "Todd Kulesza <todd@dropline.net>",
      "",
      _("With help from:"),
      "Davyd Madeley <davyd@ucc.asn.au>",
      "Grahame Bowland <grahame@angrygoats.net>",
      "Geoff King <0glk@qlink.queensu.ca>",
      "Jess Little <jackymac@gmail.com>",
      "Evan Martin <evan@livejournal.com>",
      "Margot Pearce <cupidelock@gmail.com>",
      "Ari Pollak <compwiz@aripollak.com>",
      "Isak Savo <iso01001@student.mdh.se>",
      "",
      NULL
    };
  const gchar *documentors[] = 
    {
      "Todd Kulesza <todd@dropline.net>",
      NULL
    };
  /*
   * Translators should localize the following string
   * which will give them credit in the About box.
   * E.g. "Fulano de Tal <fulano@detal.com>" */
  const gchar *translators = _("translator-credits");
 
  if (!logo)
    {
      gchar *path = g_build_filename (DATADIR, "pixmaps", "drivel-48.png", NULL);
      logo = gdk_pixbuf_new_from_file (path, NULL);
      g_free (path);
    }
  
  gtk_show_about_dialog (GTK_WINDOW (dc->current_window), 
			 "authors", authors,
			 "comments", _("A journal client for the GNOME desktop."),
			 "copyright", "2002-2005 Todd Kulesza",
			 "documenters", documentors,
			 "logo", logo,
			 "name", "Drivel",
			 "translator-credits", translators,
			 "version", VERSION,
			 "website", "http://www.dropline.net/drivel",
			 NULL);
  gtk_about_dialog_set_url_hook (url_cb, dc, NULL);

  return;
}
